#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;

#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators
{
	public class EdsLevel2Dom : Indicator
	{
		
        #region Variables
		private	List<DOMRow> askRows = new List<DOMRow>();
		private	List<DOMRow> bidRows = new List<DOMRow>();
		private List<DOMRow> oneDOMRow = null;
		private Dictionary<double, int> askRowsSorted = new Dictionary<double, int>();
		private Dictionary<double, int> bidRowsSorted = new Dictionary<double, int>();
		
		private SharpDX.RectangleF rectText;
        private SharpDX.RectangleF rect;

		private int x;
		private int y;
		private int barLength;
		private int barYOffset;
		private double barLengthScale = 1;
		private int barHeight = 1;
		private DateTime lastRefresh = DateTime.MinValue;
		private DateTime nextRefreshTime = DateTime.MinValue;

	//	private int totalAskVolume = 0;
	//	private int totalBidVolume = 0;
		private int DOMLargestVolume = 1;
		private int i = 0;
		private string rowVolumeString = "";
		
		private ScalingSchemeType scalingScheme = ScalingSchemeType.MaximumOfLadder;
		
        private double tickSize;
		private int onMarketCount;
		private int onRenderCount;
		private bool firstRound;
		private DateTime lastTestTime = DateTime.MinValue;
		private DateTime nextTestTime = DateTime.MinValue;
		private double RefreshDelayOrigonal;
		private double actualElapsedTime;
		private double rateOnMarketDepthCalls;
		private double rateOnRenderCalls;
		
		#endregion

		//EdsLevel2Dom_V2.0
		//revisions:
		//V2.0  rewrite EdsLevel2Dom_V1.0 (NT7) for NT8, added Performance Test, 2/22/2017 by Camdo
		//V2.0.1  lock keyword lacked statement-block brackets  2/24/2017 by Camdo
		
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"EdsLevel2Dom_V2.0 for NT8 by Camdo";
				Name										= "EdsLevel2Dom";
				Calculate									= Calculate.OnBarClose;
				IsOverlay									= true;
				DrawOnPricePanel							= true;
				PaintPriceMarkers							= false;
				ArePlotsConfigurable						= false;
				DisplayInDataBox							= false;
				ScaleJustification							= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				//Disable this property if your indicator requires custom values that cumulate with each new market data event. 
				//See Help Guide for additional information.
				IsSuspendedWhileInactive					= true;

				MarginRt 							= 0;
				RefreshDelay						= 300;
				HistogramOpacity 					= 100;
				ScalingNumber						= 1500;
				BarSpacing							= 0.8;
                AskColor                            = Brushes.Crimson;
				BidColor                            = Brushes.DodgerBlue;
                TextColor                        	= Brushes.White;
				PerformanceTest						= false;
				PerformanceSamplePeriod				= 60000;

			}
			else if (State == State.Configure)
			{
				tickSize = TickSize;
				firstRound = true;
			}
			else if(State == State.Historical)
			{
				//no level 2 for historical data
				return;
			}
		}

		#region On Event Updates
		protected override void OnBarUpdate()
		{
		}
		
		protected override void OnMarketDepth(MarketDepthEventArgs e)
		{
			//Sample e  note pos is the DOM ladder rung position, in ticks, from price.
			//instrument='ES 03-17 Globex' type=Bid mm='' operation=Add pos=0 price=2353.25 volume=179 time=2/20/2017 12:04:33 PM isReset=False

			++onMarketCount;		//for performance test
			
			// protect e.Instrument.MarketDepth.Asks and e.Instrument.MarketDepth.Bids against in-flight changes
			lock (e.Instrument.SyncMarketDepth)
			{			
				// Checks to see if the Market Data is of the Ask type
				if (e.MarketDataType == MarketDataType.Ask)
					oneDOMRow = askRows;
				
				// Checks to see if the Market Data is of the Bid type
				else if (e.MarketDataType == MarketDataType.Bid)
					oneDOMRow = bidRows;
				
				if (oneDOMRow == null)
					return;

				// Checks to see if the action taken by the Ask data was an insertion into the ladder
				if (e.Operation == Operation.Add)
					oneDOMRow.Insert(e.Position, new DOMRow(e.Price, (int) e.Volume));
				
				/* Checks to see if the action taken by the Ask data was a removal of itself from the ladder
				Note: Due to the multi threaded architecture of the NT core, race conditions could occur
				-> check if e.Position is within valid range */
				else if (e.Operation == Operation.Remove && e.Position < oneDOMRow.Count)
					oneDOMRow.RemoveAt(e.Position);
				
				/* Checks to see if the action taken by the Ask data was to update a data already on the ladder
				Note: Due to the multi threaded architecture of the NT core, race conditions could occur
				-> check if e.Position is within valid range */
				else if (e.Operation == Operation.Update && e.Position < oneDOMRow.Count)
				{
					oneDOMRow[e.Position].Price = e.Price;
					oneDOMRow[e.Position].Volume = (int) e.Volume;
				}
			}

			//NT8 refreshes every 250 ms by default.  Code below may actually delay the refresh by constantly pushing the time into the future.
			// if it has been longer than xxx milliseconds since the last time this method was called update the chart visually
			if (NinjaTrader.Core.Globals.Now >= nextRefreshTime)
			{
				ForceRefresh();								//resets NT timer.  Actual refresh will occur no later than 250 MS from now.
				nextRefreshTime = NinjaTrader.Core.Globals.Now.AddMilliseconds(RefreshDelay);
			}

			if(PerformanceTest)
			{
				PerformanceOutput();	
			}
		}
		#endregion
	
	
		#region Render Chart
        protected override void OnRender(ChartControl chartControl, ChartScale chartScale)
		{	
		//	totalAskVolume = 0;
		//	totalBidVolume = 0;
			
			++onRenderCount;		//for performance test
			
			// find largest volume, copy arrays, and combine depths of same price
			DOMLargestVolume = 1;
			askRowsSorted.Clear();
			bidRowsSorted.Clear();
			for (i = 0; i < askRows.Count; i++)
			{
				if (askRowsSorted.ContainsKey(askRows[i].Price) == false)
					askRowsSorted.Add(askRows[i].Price, askRows[i].Volume);
				else
					askRowsSorted[askRows[i].Price] += askRows[i].Volume;
				if (askRowsSorted[askRows[i].Price] > DOMLargestVolume)
					DOMLargestVolume = askRowsSorted[askRows[i].Price];
			}
			for (i = 0; i < bidRows.Count; i++)
			{
				if (bidRowsSorted.ContainsKey(bidRows[i].Price) == false)
					bidRowsSorted.Add(bidRows[i].Price, bidRows[i].Volume);
				else
					bidRowsSorted[bidRows[i].Price] += bidRows[i].Volume;
				if (bidRowsSorted[bidRows[i].Price] > DOMLargestVolume)
					DOMLargestVolume = bidRowsSorted[bidRows[i].Price];
			}
			
			switch (scalingScheme)
			{
				case ScalingSchemeType.MaximumOfLadder:
					barLengthScale = 120.0 / DOMLargestVolume;
					break;
				case ScalingSchemeType.FixedNumber:
					barLengthScale = 120.0 / ScalingNumber;
					break;
				default:
					barLengthScale = 120.0 / DOMLargestVolume;
					break;
			}
			
			x = ChartControl.CanvasRight - MarginRt;
			//bar height is the size of one chart tick
			float pixelForDistance = chartScale.GetPixelsForDistance(TickSize);
			barHeight = (int)((pixelForDistance) * BarSpacing);
			if (barHeight < 1)  //too small to be useful
			return;
			
			
			SharpDX.Direct2D1.Brush askBarBrushDX = AskColor.ToDxBrush(RenderTarget);
			SharpDX.Direct2D1.Brush bidBarBrushDX = BidColor.ToDxBrush(RenderTarget);
			SharpDX.Direct2D1.Brush textBrush = TextColor.ToDxBrush(RenderTarget);
			SharpDX.DirectWrite.TextFormat textFormat = new SharpDX.DirectWrite.TextFormat(NinjaTrader.Core.Globals.DirectWriteFactory, "Arial", barHeight);
			
			askBarBrushDX.Opacity = HistogramOpacity * 0.01f;
			bidBarBrushDX.Opacity = HistogramOpacity * 0.01f;
			textBrush.Opacity = HistogramOpacity * 0.01f;
			
			foreach (KeyValuePair<double, int> key in askRowsSorted)
			{
				if (key.Value > 0)
				{
				//	totalAskVolume += key.Value;
					barLength = (int) (key.Value * barLengthScale);
					
                    double priceLower = key.Key - tickSize / 2;
                    int yLower = chartScale.GetYByValue(priceLower);
                    int yUpper = chartScale.GetYByValue(priceLower + tickSize);

					rect = new SharpDX.RectangleF(x - barLength, yUpper, barLength, barHeight);
					RenderTarget.FillRectangle(rect, askBarBrushDX);
				
					//print Vol data over bar
					if (barHeight >= 5)
                    {
                        {
							string rowVolumeStr = RowVolumeString(key.Value);
							rectText = new SharpDX.RectangleF(x - barLength, yUpper, barLength, barHeight);
							RenderTarget.DrawText(rowVolumeStr, textFormat, rectText, textBrush);
                        }
                    }
				}
			}
			
			foreach (KeyValuePair<double, int> key in bidRowsSorted)
			{
				if (key.Value > 0)
				{
				//	totalBidVolume += key.Value;
					barLength = (int) (key.Value * barLengthScale);
					
                    double priceLower = key.Key - tickSize / 2;
                    int yLower = chartScale.GetYByValue(priceLower);
                    int yUpper = chartScale.GetYByValue(priceLower + tickSize);

					rect = new SharpDX.RectangleF(x - barLength, yUpper, barLength, barHeight);
					RenderTarget.FillRectangle(rect, bidBarBrushDX);
				
					//print Vol data over bar
					if (barHeight >= 5)
                    {
                        {
							string rowVolumeStr = RowVolumeString(key.Value);
							rectText = new SharpDX.RectangleF(x - barLength, yUpper, barLength, barHeight);
							RenderTarget.DrawText(rowVolumeStr, textFormat, rectText, textBrush);
                        }
                    }
				}
			}
            //Dispose of DX objects
            askBarBrushDX.Dispose();
            bidBarBrushDX.Dispose();
			textFormat.Dispose();
			textBrush.Dispose();

		}
		#endregion
	
		
		#region Support Methods
		private string RowVolumeString(int volume)
		{
			if (volume >= 1000000)
				return (volume / 1000000).ToString() + "M";
			else if (volume >= 100000)
				return (volume / 1000).ToString() + "K";
			else
				return volume.ToString();
		}
		
		private class DOMRow
		{
			public double Price;
			public int Volume;

			public DOMRow(double myPrice, int myVolume)
			{
				Price = myPrice;
				Volume = myVolume;
			}
		}
		
		//Performance Test:  Starting at 10ms Refresh delay is doubled after each test period until greater than 5 seconds
		private void PerformanceOutput()
		{
			if(firstRound)
			{
				Print("Performance Test in Progress \nTest interval = " + PerformanceSamplePeriod / 1000 + " seconds");
				Print("Test will override RefreshDelay and restore it at the end of the test.  Do not interupt the test.");
				lastTestTime = NinjaTrader.Core.Globals.Now;
				nextTestTime = NinjaTrader.Core.Globals.Now.AddMilliseconds(PerformanceSamplePeriod);
				onMarketCount = 0;
				onRenderCount = 0;
				RefreshDelayOrigonal = RefreshDelay;
				RefreshDelay = 10;		//milliseconds
				nextRefreshTime = NinjaTrader.Core.Globals.Now.AddMilliseconds(RefreshDelay);
				firstRound = false;
			}
			else if (NinjaTrader.Core.Globals.Now >= nextTestTime)
			{
				//print test results
				actualElapsedTime = NinjaTrader.Core.Globals.Now.Subtract(lastTestTime).TotalSeconds;
				rateOnMarketDepthCalls = onMarketCount / actualElapsedTime;
				rateOnRenderCalls = onRenderCount / actualElapsedTime;
				Print( NinjaTrader.Core.Globals.Now.ToString("yyyy-MM-dd    HH:mm:ss:fff")
					+ "    RefreshDelay = " + RefreshDelay + " millisec"
					+ "    actualElapsedTime = " + actualElapsedTime.ToString("N3") + " sec"
					+ "    onMarketDepth calls = " + onMarketCount
					+ "    onRender calls = " + onRenderCount
					+ "    OnMarketDepth calls/sec = " + rateOnMarketDepthCalls.ToString("N2")
					+ "    OnRender calls/sec = " + rateOnRenderCalls.ToString("N2")
					);
				
				//check if Performance test is complete
				if(RefreshDelay >= 5000)
				{
					firstRound = true;
					RefreshDelay = RefreshDelayOrigonal;
					PerformanceTest = false;
					Print("Test Complete \nRefresh Delay restored to = " + RefreshDelay + " millisec\n");
				}
				else
				{
					//setup parameters for next test increment
					onMarketCount = 0;
					onRenderCount = 0;
					RefreshDelay = RefreshDelay * 2;				//Refresh delay is doubled each round until Performance test is complete
					lastTestTime = NinjaTrader.Core.Globals.Now;
					nextTestTime = NinjaTrader.Core.Globals.Now.AddMilliseconds(PerformanceSamplePeriod);
				}
			}
		}
		//test results show 300 ms is optimal RefreshDelay
		//faster times result in less calls to onRender() per second
		//Test results: 
		//					RefreshDelay	onRenders/sec	onMarketDepth()/sec
		//					5000			1.38			62.23
		//					300				2.16			57.50
		//					100				2.11			27.05

		
		
		
		
		
		
		#endregion

        #region Properties

		
        [Range(0, 100), NinjaScriptProperty]
		[Display(Name="Histogram opacity", Description="Histogram opacity percentage. (0-100)", GroupName="01 Plots", Order = 1)]
		public int HistogramOpacity
		{ get; set; }
		
		[Range(10, int.MaxValue),NinjaScriptProperty]
		[Display(Name="Refresh Delay", Description="histogram repaint delay in milliseconds for slow updating charts", GroupName="01 Plots", Order = 2)]
		public double RefreshDelay
		{ get; set; }
		
		[Range(0, 1000), NinjaScriptProperty]
		[Display(Name="Right Margin", Description="How much space to the right of the histogram (in pixels)", GroupName="01 Plots", Order = 3)]
		public int MarginRt
		{ get; set; }
		
		[NinjaScriptProperty]
		[Display(Name="Scaling Scheme", Description="", GroupName="01 Plots", Order = 4)]
		public ScalingSchemeType ScalingScheme
		{
			get { return scalingScheme;}
			set {scalingScheme = value;}
		}
		
		[Range(1, int.MaxValue), NinjaScriptProperty]
		[Display(Name="Scaling Number", Description="", GroupName="01 Plots", Order = 5)]
		public int ScalingNumber
		{ get; set; }

		[Range(0.1, 1), NinjaScriptProperty]
		[Display(Name="Bar Spacing", Description="A factor of tick size 0.1 - 1.0", GroupName="01 Plots", Order = 6)]
		public double BarSpacing
		{ get; set; }

        [XmlIgnore]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Ask color", Description = "Color for the Ask bar.", GroupName = "02 Colors", Order = 6)]
        public System.Windows.Media.Brush AskColor
        { get; set; }
        [Browsable(false)]
        public string AskColorSerialize
        {
            get { return Serialize.BrushToString(AskColor); }
            set { AskColor = Serialize.StringToBrush(value); }
        }

        [XmlIgnore]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Bid color", Description = "Color for the Bid bar.", GroupName = "02 Colors", Order = 8)]
        public System.Windows.Media.Brush BidColor
        { get; set; }
        [Browsable(false)]
        public string BidColorSerialize
        {
            get { return Serialize.BrushToString(BidColor); }
            set { BidColor = Serialize.StringToBrush(value); }
        }
		
        [XmlIgnore]
        [Display(ResourceType = typeof(Custom.Resource), Name = "Text color", Description = "Color for the bar text.", GroupName = "02 Colors", Order = 9)]
        public System.Windows.Media.Brush TextColor
        { get; set; }
        [Browsable(false)]
        public string TextColorSerialize
        {
            get { return Serialize.BrushToString(TextColor); }
            set { TextColor = Serialize.StringToBrush(value); }
        }

		[NinjaScriptProperty]
		[Display(Name="Test Performance", Description="Performance test to evaluate Refresh Time.  Starting at 10ms Refresh delay is doubled after each test period until greater than 5 seconds", GroupName="03 Performance Test", Order = 10)]
		public bool PerformanceTest
		{ get; set; }
		
		[Range(1000, 300000),NinjaScriptProperty]
		[Display(Name="Sample Period ms", Description="Data colection period for performance sample in milliseconds (1000 - 300000) 60000 = 1 minute", GroupName="03 Performance Test", Order = 11)]
		public int PerformanceSamplePeriod
		{ get; set; }
		
		
        #endregion

		
	}
}

public enum ScalingSchemeType
{
	MaximumOfLadder =0,
	FixedNumber =1,
}





#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private EdsLevel2Dom[] cacheEdsLevel2Dom;
		public EdsLevel2Dom EdsLevel2Dom(int histogramOpacity, double refreshDelay, int marginRt, ScalingSchemeType scalingScheme, int scalingNumber, double barSpacing, bool performanceTest, int performanceSamplePeriod)
		{
			return EdsLevel2Dom(Input, histogramOpacity, refreshDelay, marginRt, scalingScheme, scalingNumber, barSpacing, performanceTest, performanceSamplePeriod);
		}

		public EdsLevel2Dom EdsLevel2Dom(ISeries<double> input, int histogramOpacity, double refreshDelay, int marginRt, ScalingSchemeType scalingScheme, int scalingNumber, double barSpacing, bool performanceTest, int performanceSamplePeriod)
		{
			if (cacheEdsLevel2Dom != null)
				for (int idx = 0; idx < cacheEdsLevel2Dom.Length; idx++)
					if (cacheEdsLevel2Dom[idx] != null && cacheEdsLevel2Dom[idx].HistogramOpacity == histogramOpacity && cacheEdsLevel2Dom[idx].RefreshDelay == refreshDelay && cacheEdsLevel2Dom[idx].MarginRt == marginRt && cacheEdsLevel2Dom[idx].ScalingScheme == scalingScheme && cacheEdsLevel2Dom[idx].ScalingNumber == scalingNumber && cacheEdsLevel2Dom[idx].BarSpacing == barSpacing && cacheEdsLevel2Dom[idx].PerformanceTest == performanceTest && cacheEdsLevel2Dom[idx].PerformanceSamplePeriod == performanceSamplePeriod && cacheEdsLevel2Dom[idx].EqualsInput(input))
						return cacheEdsLevel2Dom[idx];
			return CacheIndicator<EdsLevel2Dom>(new EdsLevel2Dom(){ HistogramOpacity = histogramOpacity, RefreshDelay = refreshDelay, MarginRt = marginRt, ScalingScheme = scalingScheme, ScalingNumber = scalingNumber, BarSpacing = barSpacing, PerformanceTest = performanceTest, PerformanceSamplePeriod = performanceSamplePeriod }, input, ref cacheEdsLevel2Dom);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.EdsLevel2Dom EdsLevel2Dom(int histogramOpacity, double refreshDelay, int marginRt, ScalingSchemeType scalingScheme, int scalingNumber, double barSpacing, bool performanceTest, int performanceSamplePeriod)
		{
			return indicator.EdsLevel2Dom(Input, histogramOpacity, refreshDelay, marginRt, scalingScheme, scalingNumber, barSpacing, performanceTest, performanceSamplePeriod);
		}

		public Indicators.EdsLevel2Dom EdsLevel2Dom(ISeries<double> input , int histogramOpacity, double refreshDelay, int marginRt, ScalingSchemeType scalingScheme, int scalingNumber, double barSpacing, bool performanceTest, int performanceSamplePeriod)
		{
			return indicator.EdsLevel2Dom(input, histogramOpacity, refreshDelay, marginRt, scalingScheme, scalingNumber, barSpacing, performanceTest, performanceSamplePeriod);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.EdsLevel2Dom EdsLevel2Dom(int histogramOpacity, double refreshDelay, int marginRt, ScalingSchemeType scalingScheme, int scalingNumber, double barSpacing, bool performanceTest, int performanceSamplePeriod)
		{
			return indicator.EdsLevel2Dom(Input, histogramOpacity, refreshDelay, marginRt, scalingScheme, scalingNumber, barSpacing, performanceTest, performanceSamplePeriod);
		}

		public Indicators.EdsLevel2Dom EdsLevel2Dom(ISeries<double> input , int histogramOpacity, double refreshDelay, int marginRt, ScalingSchemeType scalingScheme, int scalingNumber, double barSpacing, bool performanceTest, int performanceSamplePeriod)
		{
			return indicator.EdsLevel2Dom(input, histogramOpacity, refreshDelay, marginRt, scalingScheme, scalingNumber, barSpacing, performanceTest, performanceSamplePeriod);
		}
	}
}

#endregion
